/* 

 Copyright(C) 2006 Osamu Matsumoto <osamu-m@is.naist.jp>

 This is free software with ABSOLUTELY NO WARRANTY.
 You can redistribute it and/or modify it under the terms of
 the GNU General Public License version 2.

 Original Source code is part of CARMEN(Carnegie Mellon Robot 
 Navigation Toolkit) 

 CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 Roy, Sebastian Thrun, Dirk Haehnel, Cyrill Stachniss,
 and Jared Glover

 2008.11.20  This source code is editted by Tsuyoshi Suenaga, NAIST
             <tsuyo-s@is.naist.jp>

*/

#include <stdio.h>
#include <string.h>
#include "sick.h"

void sick_set_default_parameters(sick_laser_p laser, int laser_num)
{
  laser->settings.type = LMS;
  laser->settings.range_res = CM;
  laser->settings.range_dist = RANGE80M;
  laser->settings.laser_num = laser_num;
  strcpy(laser->settings.device_name, "/dev/ttyS0");
  laser->settings.detect_baudrate = TRUE;
  laser->settings.use_highspeed = FALSE;
  laser->settings.start_baudrate = 9600;
  laser->settings.set_baudrate = 38400;
  laser->settings.databits = 8;
  laser->settings.parity = N;
  laser->settings.stopbits = 1;
  laser->settings.hwf = 0;
  laser->settings.swf = 0;
  laser->settings.angle_range = 180;
//  laser->settings.angle_resolution = RES_1_00_DEGREE;
  laser->settings.angle_resolution = RES_0_50_DEGREE;
  laser->settings.laser_flipped = 0;
  laser->settings.use_remission = 0;
}

int check_parameter_settings(sick_laser_p laser)
{
  /*********************** TYPE CHECKING **************************/
  if(laser->settings.type == PLS) {
    strncpy((char *)laser->settings.password, (const char *)PLS_PASSWORD, 8);
    laser->settings.parity = E;
  } 
  if(laser->settings.type == LMS) {
    strncpy((char *)laser->settings.password, (const char *)LMS_PASSWORD, 8);
    laser->settings.parity = N;
  } 
  
  /*********************** START BAUDRATE **************************/
  if(laser->settings.detect_baudrate)
    laser->settings.start_baudrate = 9600;
  else if(laser->settings.start_baudrate != 9600 &&
	  laser->settings.start_baudrate != 19200 &&
	  laser->settings.start_baudrate != 38400 &&
	  laser->settings.start_baudrate != 500000) {
    fprintf(stderr, "ERROR: start baudrate = %d is not valid!\n",
	    laser->settings.start_baudrate);
    return FALSE;
  }

  /*********************** SET BAUDRATE **************************/
  if(laser->settings.set_baudrate != 9600 &&
     laser->settings.set_baudrate != 19200 &&
     laser->settings.set_baudrate != 38400 &&
     laser->settings.set_baudrate != 500000) {
    fprintf(stderr, "ERROR: set baudrate = %d is not valid!\n",
	    laser->settings.set_baudrate);
    return FALSE;
  } 
  else if(laser->settings.set_baudrate == 500000)
    laser->settings.use_highspeed = TRUE;

  /*********************** NUM VALUES **************************/
  if(laser->settings.angle_range != 180 && 
     laser->settings.angle_range != 100) {
    fprintf(stderr, "ERROR: angle range = %d is not valid!\n",
	    laser->settings.angle_range);
    return FALSE;
  }
  
  /************************** ANGLE RANGE ************************/
  if(laser->settings.angle_range == 100) {
    if(laser->settings.angle_resolution == RES_1_00_DEGREE)
      laser->settings.num_values = 101;
    else if(laser->settings.angle_resolution == RES_0_50_DEGREE)
      laser->settings.num_values = 201;
    else if(laser->settings.type == LMS)
      laser->settings.num_values = 401;
    else
      fprintf(stderr, "ERROR: ang-res=0.25 is not valid for this laser!\n");
  }
  else {
    if(laser->settings.angle_resolution == RES_1_00_DEGREE)
      laser->settings.num_values = 181;
    else if(laser->settings.angle_resolution == RES_0_50_DEGREE)
      laser->settings.num_values = 361;
    else {
      fprintf(stderr, "ERROR: ang-res=0.25 and ang-range=180 is not valid!\n");
      return FALSE;
    }
  }
  if(laser->settings.type == PLS) {
    if(laser->settings.angle_range == 100) {
      fprintf(stderr, "ERROR: type = PLS and ang-range=100 is not valid!\n");
      return FALSE;
    } 
  }


  /********************** REMISSION RANGE ************************/
  /* remission values - start */
  if (laser->settings.use_remission == 1 && 
      laser->settings.type != LMS) {
    fprintf(stderr, "ERROR: remission values are only available using LMS laser!\n");
    return FALSE;
  }
  if (laser->settings.use_remission == 1 && 
      laser->settings.angle_resolution != RES_1_00_DEGREE) {
    fprintf(stderr, "ERROR: remission values are only available with 1.0 degree resolution!\n");
    return FALSE;
  }

  if(laser->settings.use_remission == 1) 
    laser->settings.rem_values = laser->settings.num_values;
  else 
    laser->settings.rem_values = 0;
  /* remission values - stop */

  return TRUE;
}


int sick_parse_params(sick_laser_p laser, const char *dev, const char *type, 
		       double res, const char *rem, double fov,int speed)
{
  strcpy(laser->settings.device_name, dev);
  if(strcmp(type, "LMS") == 0)
    laser->settings.type = LMS;
  else if(strcmp(type, "PLS") == 0)
    laser->settings.type = PLS;
  
  if (fabs(fov-M_PI) < 0.1 || fabs(fov-100.0/180.0*M_PI) < 0.1){
       fprintf(stderr,"The parameter laser_laserX_fov in the ini file must\nbe specified in degrees not in radians!\n");
       return FALSE;
  }
  laser->settings.angle_range = (int)fov;
  
  if ( laser->settings.angle_range != 180 && 
       laser->settings.angle_range != 100 ){
       fprintf(stderr,"The laser driver only provides 180 deg and 100 deg field of view!\n");
       return FALSE;
  }
  
  if(res == 0.25) {
    laser->settings.angle_resolution = RES_0_25_DEGREE;
    laser->settings.angle_range = 100;
  }
  else if(res == 0.5)
    laser->settings.angle_resolution = RES_0_50_DEGREE;
  else
    laser->settings.angle_resolution = RES_1_00_DEGREE;

  /* remission values - start */
  if(strcmp(rem, "direct") == 0) {
    laser->settings.use_remission = 1;
    laser->settings.range_dist = REMISSION_DIRECT;
  }
  else if(strcmp(rem, "normalized") == 0) {
    laser->settings.use_remission = 1;
    laser->settings.range_dist = REMISSION_NORM;
  }
  else if(strcmp(rem, "no") == 0) {
    laser->settings.use_remission = 0;
  }
  else if(strcmp(rem, "off") == 0) {
    laser->settings.use_remission = 0;
    fprintf(stderr,"Warning: please set the value of the parameter \nlaser_use_remission to \"no\" and do not use \"off\".\nAssuming \"no\" for now.\n\n");
  }
  else {
       fprintf(stderr,"ERROR: Parameter laser_use_remission for laser %d has invalid value: %s\nPossible values are: direct, normalized and off.\n", laser->settings.laser_num, rem);
       return FALSE;
  }
  
  /* remission values - stop */
  
  /* speed */
  laser->settings.set_baudrate = speed;

  if(!check_parameter_settings(laser)) {
    return FALSE;
  }

  return TRUE;
}
