/* 

 Copyright(C) 2006 Osamu Matsumoto <osamu-m@is.naist.jp>

 This is free software with ABSOLUTELY NO WARRANTY.
 You can redistribute it and/or modify it under the terms of
 the GNU General Public License version 2.

 Original Source code is part of CARMEN(Carnegie Mellon Robot 
 Navigation Toolkit) 

 CARMEN Copyright (c) 2002 Michael Montemerlo, Nicholas
 Roy, Sebastian Thrun, Dirk Haehnel, Cyrill Stachniss,
 and Jared Glover

*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>
#include <sys/time.h>
#include <linux/serial.h>


#define READ_TIMEOUT          250000      /* less than 1e6 */
#define _POSIX

int wrap_serial_writen(int dev_fd, unsigned char *buf, int nChars)
{
  int amountWritten = 0;
  
  while(nChars > 0) {
    amountWritten = write(dev_fd, buf, nChars);
    if(amountWritten < 0) {
      if(errno == EWOULDBLOCK) {
        fprintf(stderr, "\nWARNING: writeN: EWOULDBLOCK: trying again!\n");
      } 
      else {
        return -1;
      }
    }
    else {
      nChars -= amountWritten;
      buf += amountWritten;
    }
  }
  return 0;
}

int wrap_serial_readn(int dev_fd, unsigned char *buf, int nChars)
{
  int amountRead = 0, bytes_read = 0;
  struct timeval t;
  fd_set set;
  int err;

  while(nChars > 0) {
    t.tv_sec = 0;
    t.tv_usec = READ_TIMEOUT;
    FD_ZERO(&set);
    FD_SET(dev_fd, &set);
    err = select(dev_fd + 1, &set, NULL, NULL, &t);
    if(err == 0)
      return -2;

    amountRead = read(dev_fd, buf, nChars);
    if(amountRead < 0 && errno != EWOULDBLOCK)
      return -1;
    else if(amountRead > 0) {
      bytes_read += amountRead;
      nChars -= amountRead;
      buf += amountRead;
    }
  }
  return bytes_read;
}

long wrap_serial_numChars(int dev_fd)
{
  long available = 0;
  
  if(ioctl(dev_fd, FIONREAD, &available) == 0)
    return available;
  else
    return -1;
}

int wrap_serial_set_low_latency(int fd){

  struct serial_struct serial; 
  int result;
  result=ioctl(fd, TIOCGSERIAL, &serial);
  
  if (result) {
/*     carmen_warn("libcarmenserial: Cannot get the serial attributes for low latency serial mode.\n Switching to normal mode\n"); */
    return result;
  } else {
    serial.flags |= ASYNC_LOW_LATENCY;
    serial.xmit_fifo_size = 1;
    ioctl(fd, TIOCSSERIAL, &serial); 
    if (result){
/*       carmen_warn("libcarmenserial: Cannot activeate low latency mode\nSwitching to normal mode\n"); */
      return result;
    }
  }
  return result;
}

int wrap_serial_ClearInputBuffer(int dev_fd)
{
  int max_serial_buffer_size = 16384; 
  unsigned char buffer[max_serial_buffer_size]; 
  int val = 0;
  int val_total = 0;
  
  val_total = wrap_serial_numChars(dev_fd);
  val = val_total;
  
  while (val > max_serial_buffer_size) {
    read(dev_fd, &buffer, max_serial_buffer_size);
    val -= max_serial_buffer_size;
  }
  
  if(val > 0) 
    read(dev_fd, &buffer, val);
  
  return(val_total);
}

